" nvim-lspconfig
sign define DiagnosticSignError text=✗ texthl=DiagnosticSignError linehl= numhl=
sign define DiagnosticSignWarn text=⚠ texthl=DiagnosticSignWarn linehl= numhl=
sign define DiagnosticSignInfo text=ℹ texthl=DiagnosticSignInfo linehl= numhl=
sign define DiagnosticSignHint text= texthl=DiagnosticSignHint linehl= numhl=

set completeopt=menuone,noinsert,noselect

lua << EOF
lsp_mappings = function(bufnr)
  local function buf_set_keymap(...) vim.api.nvim_buf_set_keymap(bufnr, ...) end

  -- Mappings.
  local opts = { noremap=true, silent=true }
  buf_set_keymap("n", "gD", "<cmd>lua vim.lsp.buf.declaration()<CR>", opts)
  buf_set_keymap("n", "gd", "<cmd>lua vim.lsp.buf.definition()<CR>", opts)
  buf_set_keymap("n", "K", "<cmd>lua vim.lsp.buf.hover()<CR>", opts)
  buf_set_keymap("n", "gi", "<cmd>lua vim.lsp.buf.implementation()<CR>", opts)
  buf_set_keymap("n", "<c-k>", "<cmd>lua vim.lsp.buf.signature_help()<CR>", opts)

  buf_set_keymap("n", "<space>wa", "<cmd>lua vim.lsp.buf.add_workspace_folder()<CR>", opts)
  buf_set_keymap("n", "<space>wr", "<cmd>lua vim.lsp.buf.remove_workspace_folder()<CR>", opts)
  buf_set_keymap("n", "<space>wl", "<cmd>lua print(vim.inspect(vim.lsp.buf.list_workspace_folders()))<CR>", opts)
  buf_set_keymap("n", "<space>D", "<cmd>lua vim.lsp.buf.type_definition()<CR>", opts)
  buf_set_keymap("n", "<space>rn", "<cmd>lua vim.lsp.buf.rename()<CR>", opts)

  buf_set_keymap("n", "gr", "<cmd>lua vim.lsp.buf.references()<CR>", opts)
  buf_set_keymap("n", "g0", "<cmd>lua vim.lsp.buf.document_symbol()<CR>", opts)
  buf_set_keymap("n", "gW", "<cmd>lua vim.lsp.buf.workspace_symbol()<CR>", opts)

  buf_set_keymap("n", "<space>e", "<cmd>lua vim.diagnostic.open_float()<CR>", opts)
  buf_set_keymap("n", "[d", "<cmd>lua vim.diagnostic.goto_prev()<CR>", opts)
  buf_set_keymap("n", "]d", "<cmd>lua vim.diagnostic.goto_next()<CR>", opts)
  buf_set_keymap("n", "<space>q", "<cmd>lua vim.diagnostic.setloclist()<CR>", opts)
end
EOF

" lspkind-nvim
lua cmp_format = require("lspkind").cmp_format {
\   mode = "symbol",
\   preset = "codicons",
\   menu = {nvim_lsp = "[LSP]", buffer = "[Buffer]", path = "[Path]"}
\ }

lua << EOF
local nvim_lsp = require("lspconfig")

local on_attach = function(client, bufnr)
  vim.api.nvim_command("autocmd CursorHoldI <buffer> lua vim.diagnostic.open_float({show_header = false})")
  vim.lsp.handlers["textDocument/publishDiagnostics"] = vim.lsp.with(
    vim.lsp.diagnostic.on_publish_diagnostics, {
      update_in_insert = true,
      virtual_text = false
    }
  )

  local cmp = require("cmp")
  cmp.setup {
    mapping = {
      ["<C-n>"] = cmp.mapping.select_next_item { behavior = cmp.SelectBehavior.Insert },
      ["<C-p>"] = cmp.mapping.select_prev_item { behavior = cmp.SelectBehavior.Insert },
    },
    formatting = {format = cmp_format},
    sources = {
      {name = "nvim_lsp"},
      {name = "buffer", keyword_length = 5},
      {name = "path"}
    },
    experimental = {ghost_text = true},
  }
  lsp_mappings(bufnr)
end

local capabilities = vim.lsp.protocol.make_client_capabilities()
capabilities = require("cmp_nvim_lsp").update_capabilities(capabilities)

local settings = require("lsp_server_settings")
local servers = {"pylsp", "clangd"}
for _, lsp in ipairs(servers) do
  nvim_lsp[lsp].setup {on_attach=on_attach, settings={[lsp]=settings[lsp]},
                       capabilities=capabilities}
end
EOF

" nvim-cmp
autocmd FileType gitcommit lua require("cmp").setup.buffer {
\   sources = {
\     { name = "spell" },
\     { name = "buffer", keyword_length = 5 },
\     { name = "path" },
\   },
\ }
